package com.kexun.annotation;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Description: 自定义注解用来标记哪些修改操作需要记录到历史日志中去。
 *              该注解需要标记到controller层，标记在controller层的目的是能够获取到页面传递过来的值和需要修改的主键ID
 *
 * @author lgs
 * @date 2023/05/16
 */
@Target(ElementType.METHOD)
@Retention(RetentionPolicy.RUNTIME)
public @interface HisModifyLog {

    /**
     * 业务类型
     * @return
     */
    String businessType() default "";

    /**
     * 变更模块名称
     * @return
     */
    String module() default "";

    /**
     * 查询修改之前调用的service的bean
     * @return
     */
    String serviceBean() default "";

    /**
     * 查询修改之前需要调用的方法
     * @return
     */
    String queryMethod() default "";

    /**
     * 查询参数的参数类型 例如：Long,String等等
     * @return
     */
    String paramType() default "";

    /**
     * 从方法参数实体中解析出来的参数作为查询的ID
     * @return
     */
    String paramKey() default "";

    /**
     * 是否为批量类型的操作,主要是从页面上传递过来批量修改的数据
     * @return
     */
    boolean paramIsArray() default false;

}
