package com.kexun.service.xml;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.annotation.JSONField;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.kexun.constant.BaseConsts;
import com.kexun.constant.XmlNodeConsts;
import com.kexun.entity.FieldDefineEntity;
import com.kexun.entity.FieldParamEntity;
import com.kexun.entity.IBaseEntity;
import com.kexun.entity.ReportInfo;
import com.kexun.factory.HandleXmlFactory;
import com.kexun.factory.HandleXmlFactoryBean;
import com.kexun.model.dto.CommonParam;
import com.kexun.model.dto.ReportParseDTO;
import com.kexun.model.xml.HeadInfo;
import com.kexun.model.xml.IXmlBase;
import com.kexun.service.CrHeadInfService;
import com.kexun.service.FieldDefineService;
import com.kexun.service.FieldParamService;
import com.kexun.utils.ParseXmlUtils;
import com.kexun.utils.SpringContextUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Description:解析xml处理类
 *
 * @author lgs
 * @date 2023/05/25
 */
@Service
@Slf4j
public class ParseHandleService {

    @Autowired
    HandleXmlFactory handleXmlFactory;
    @Autowired
    CrHeadInfService crHeadInfService;
    @Autowired
    FieldDefineService fieldDefineService;
    @Autowired
    FieldParamService fieldParamService;



    /**
     * 解析处理xml的内容
     *
     * @param xmlContent xml报文的内容
     */
    public ReportParseDTO parseHandle(ReportInfo reportInfo,String xmlContent) throws Exception {
        //把xml转成Json格式，和XML保持一致的格式
        JSONObject jsonObject = ParseXmlUtils.xmlToJson(xmlContent);
        //定义Map 存放JSON
        Map<String, Object> jsonMap = new HashMap<>(16);
        //定义产品关联字段的存放JSON
        Map<String, Object> fieldJsonMap = new HashMap<>(16);
        log.info("转出json后={}", jsonObject.toJSONString());
        ReportParseDTO reportParseDTO = new ReportParseDTO();
        /**
         * 处理思路：
         * 1、根据json的key值，获取对应的Json信息段
         * 2、有了key值，识别对应的需要转成的Bean
         * 3、找到对应的service，进行各个表的保存
         */
        //循环json的节点，然后根据json各个节点进行处理
        if (!jsonObject.isEmpty()) {
            //获取HEADBO的数据，需要得到用户的相关信息，循环其他的时候，忽略掉即可
            JSONObject headJsonObject = (JSONObject) jsonObject.get(XmlNodeConsts.HEADBO);
            CommonParam commonParam = new CommonParam();
            handleCommParam(headJsonObject, commonParam);
            BeanUtils.copyProperties(commonParam,reportParseDTO);
            for (Map.Entry<String, Object> entry : jsonObject.entrySet()) {
                if (entry.getKey().equals(XmlNodeConsts.HEADBO)) {
                    continue;
                }
                String nodeName = entry.getKey();
                log.info("开始处理节点名称{}的解析工具", nodeName);
                //根据nodeName获取
                HandleXmlFactoryBean factoryBean = handleXmlFactory.getFactoryBean(nodeName);
                if (null != factoryBean) {
                    //得到xml报文信息段对应的JavaBean
                    Class<? extends IXmlBase> xmlModelBean = factoryBean.getModelBean();
                    IXmlBase IXmlBase = xmlModelBean.newInstance();
                    String json;
                    if (entry.getValue()!=null && !entry.getValue().equals("")){
                        if (entry.getValue() instanceof JSONArray){
                            json = ((JSONArray) entry.getValue()).toJSONString();
                        }else {
                            json = ((JSONObject) entry.getValue()).toJSONString();
                        }
                        //json转化成JavaBean
                        IXmlBase jsonBean = JSONObject.parseObject(json, IXmlBase.getClass());
                        IHandleXmlBaseService reportXmlBaseService = factoryBean.getReportXmlBaseService();
                        //处理简单类型的
                        if (BaseConsts.NODE_TYPE_NORMAL.equals(factoryBean.getNodeType())) {
                            //赋值
                            Class<? extends IBaseEntity> baseEntity = factoryBean.getBaseEntity();
                            IBaseEntity iBaseEntity = baseEntity.newInstance();
                            BeanUtils.copyProperties(jsonBean, iBaseEntity);
                            if (factoryBean.getSpecService() != null) {
                                Map<String, String> paramMap = factoryBean.getParamMap();
                                commonParam.setNodeMap(paramMap);
                                factoryBean.getSpecService().handleXmlSpec(commonParam, jsonBean, iBaseEntity);
                            }
                            //先查询,根据rpt_no查询是否存在
                            QueryWrapper queryWrapper = new QueryWrapper();
                            queryWrapper.eq("rpt_no", commonParam.getRptNo());
                            //根据rpt_no查询是否存在，如果存在则更新
                            Object obj = factoryBean.getService().getOne(queryWrapper);
                            if (obj!=null){
                                Class<?> aClass = obj.getClass();
                                Class<?> superclass = aClass.getSuperclass();
                                Field field = superclass.getDeclaredField("id");
                                field.setAccessible(true);
                                Object id = field.get(obj);

                                Class<? extends IBaseEntity> aClass1 = iBaseEntity.getClass();
                                Class<?> superclass1 = aClass1.getSuperclass();
                                Field idField = superclass1.getDeclaredField("id");
                                idField.setAccessible(true);
                                // 设置name字段的新值
                                idField.set(iBaseEntity, id);
                            }
                            //保存
                            factoryBean.getService().saveOrUpdate(iBaseEntity);
                            //构建json
                            if (factoryBean.getBaseFieldPackageJsonService() != null) {
                                factoryBean.getBaseFieldPackageJsonService().packageJson(reportInfo,jsonMap,fieldJsonMap, json, factoryBean.getXmlNodeName(), factoryBean.getNodeType());
                            }
                        } else if (BaseConsts.NODE_TYPE_COMPOSE.equals(factoryBean.getNodeType())) {
                            //处理复杂类型
                            Map<String, String> paramMap = factoryBean.getParamMap();
                            commonParam.setNodeMap(paramMap);
                            reportXmlBaseService.handleXmlSgmt(commonParam, jsonBean);
                            //构建json
                            if (factoryBean.getBaseFieldPackageJsonService() != null) {
                                factoryBean.getBaseFieldPackageJsonService().packageJson(reportInfo,jsonMap,fieldJsonMap, json, factoryBean.getXmlNodeName(), factoryBean.getNodeType());
                            }
                        } else if (BaseConsts.NODE_TYPE_LIST.equals(factoryBean.getNodeType())) {
                            Map<String, String> paramMap = factoryBean.getParamMap();
                            Class<? extends IXmlBase> aClass = jsonBean.getClass();
                            Field[] declaredFields = aClass.getDeclaredFields();
                            for (Field field : declaredFields) {
                                field.setAccessible(true);
                                Class<?> type = field.getType();
                                if (type.equals(List.class)) {
                                    String nodeListName = paramMap.get(BaseConsts.NODE_LIST_NAME);
                                    //获取注解标记的名字
                                    String name = field.getDeclaredAnnotation(JSONField.class).name();
                                    if (nodeListName.equals(name)) {
                                        List list = (List) field.get(jsonBean);
                                        List<IBaseEntity> iBaseEntities = new ArrayList<>();
                                        for (Object object : list) {
                                            //object是json转成JavaBean,需要把赋值给Entity,然后保存
                                            Class<? extends IBaseEntity> baseEntity = factoryBean.getBaseEntity();
                                            IBaseEntity iBaseEntity = baseEntity.newInstance();
                                            BeanUtils.copyProperties(object, iBaseEntity);
                                            //处理其他的赋值
                                            if (factoryBean.getSpecService() != null) {
                                                factoryBean.getSpecService().handleXmlSpec(commonParam, jsonBean, iBaseEntity);
                                            }
                                            iBaseEntities.add(iBaseEntity);
                                        }
                                        factoryBean.getService().saveBatch(iBaseEntities);
                                        if (factoryBean.getBaseFieldPackageJsonService() != null) {
                                            factoryBean.getBaseFieldPackageJsonService().packageJson(reportInfo,jsonMap,fieldJsonMap, json, factoryBean.getXmlNodeName(), factoryBean.getNodeType());
                                        }
                                    }
                                }
                            }
                        }
                    }else {
                        log.info("处理节点名称{}的解析工具,获取的节点值为空", nodeName);
                    }
                    log.info("处理节点名称{}的解析工具结束", nodeName);
                } else {
                    log.error("获取节点{}对应的处理Bean为空", nodeName);
                }
            }
            /**
             * 实现思路：
             * 1、直接从数据库里获取配置的衍生字段，按照json的key值去做分组，然后进行循环。
             * 2、通过配置和参数，进行计算，放到配置对应的位置去。
             */
            List<FieldDefineEntity> fieldList = fieldDefineService.queryFieldDefineByProd(reportInfo.getProductNum(),"","2");
            //按照Jsonkey进行分组
            Map<String, List<FieldDefineEntity>> jsonKeyMap = fieldList.stream()
                    .filter((e)->!StringUtils.isEmpty(e.getJsonKey()))
                    .collect(Collectors.groupingBy((e) -> e.getJsonKey()));
            jsonKeyMap.forEach((k, v) -> {
                //key记录的是放到JSON里面去
                String key = k.toString();
                //循环衍生字段
                for (FieldDefineEntity fieldDefineEntity : v) {
                    String oneLevel = fieldDefineEntity.getOneLevel();
                    List<FieldParamEntity> fieldParamEntities = fieldParamService.listByFieldNo(fieldDefineEntity.getFieldNo());
                    //jsonKey:最终的字段需要放到JSON节点
                    String jsonKey = fieldDefineEntity.getJsonKey();
                    //获取配置的spring bean和方法
                    String formulaClass = fieldDefineEntity.getFormulaClass();
                    String formulaMethod = fieldDefineEntity.getFormulaMethod();

                    Object object = SpringContextUtil.getBean(formulaClass);
                    String name = fieldDefineEntity.getName();
                    Map<String, Object> fieldMap = new HashMap<>();
                    Object invokeResult = null;
                    Class<?> aClass = object.getClass();
                    if (!StringUtils.isEmpty(oneLevel)) {
                        HandleXmlFactoryBean factoryBean = handleXmlFactory.getFactoryBean(oneLevel);
                        Class<? extends IXmlBase> xmlModelBean = factoryBean.getModelBean();
                        try {
                            IXmlBase IXmlBase = xmlModelBean.newInstance();
                            if (jsonObject.get(oneLevel) != null) {
                                JSONObject jsonObject1 = (JSONObject) jsonObject.get(oneLevel);
                                String json = jsonObject1.toJSONString();
                                //json转化成JavaBean
                                IXmlBase jsonBean = JSONObject.parseObject(json, IXmlBase.getClass());
                                //根据衍生字段字段编码，查询该字段所需要的参数 field_param
                                Method method = aClass.getMethod(formulaMethod, List.class, FieldDefineEntity.class, IXmlBase.class);
                                invokeResult = method.invoke(object, fieldParamEntities, fieldDefineEntity, jsonBean);
                            }
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    } else {
                        Method method = null;
                        try {
                            method = aClass.getMethod(formulaMethod, List.class, FieldDefineEntity.class, JSONObject.class);
                            invokeResult = method.invoke(object, fieldParamEntities, fieldDefineEntity, jsonObject);
                        } catch (Exception e) {
                           // e.printStackTrace();
                            log.error("调用衍生字段：{}方法出现错误,错误内容为:{}",formulaMethod,e.getMessage());
                        }
                    }
                    if (invokeResult != null) {
                        fieldMap.put(name, invokeResult);
                        if (fieldJsonMap.containsKey(jsonKey)) {
                            Map map = (Map) fieldJsonMap.get(jsonKey);
                            map.putAll(fieldMap);
                            fieldJsonMap.put(jsonKey, map);
                        } else {
                            Map<String, Object> subJsonMap = new HashMap<>();
                            subJsonMap.put(jsonKey, fieldMap);
                            fieldJsonMap.putAll(subJsonMap);
                        }
                    }
                }
            });
            log.info("解析出来的基础字段JSON：{}", JSONObject.toJSONString(jsonMap));

        } else {
            log.error("xml转Json得到的JSONObject为空");
        }
        //基础字段的JSON
        reportParseDTO.setParseJson(JSONObject.toJSONString(jsonMap));
        reportParseDTO.setPrdJson(JSONObject.toJSONString(fieldJsonMap));
        return reportParseDTO;
    }

    /**
     * 处理公共的参数
     *
     * @param headJsonObject
     * @param commonParam
     */
    private void handleCommParam(JSONObject headJsonObject, CommonParam commonParam) {
        //被查询者姓名
        String name = String.valueOf(headJsonObject.get(BaseConsts.NAME));
        //被查询者证件类型代码
        String creTypeCode = String.valueOf(headJsonObject.get(BaseConsts.CRETYPECODE));
        //被查询者证件类型描述
        String creTypeDesc = String.valueOf(headJsonObject.get(BaseConsts.CRETYPEDESC));
        //被查询者证件号码
        String creNumber = String.valueOf(headJsonObject.get(BaseConsts.CRENUMBER));
        //报告编号
        String rptNo = String.valueOf(headJsonObject.get(BaseConsts.RPTNO));
        //构建公共参数
        commonParam.setName(name);
        commonParam.setIdTypeCode(creTypeCode);
        commonParam.setIdTypeDesc(creTypeDesc);
        commonParam.setIdNum(creNumber);
        commonParam.setRptNo(rptNo);
        HeadInfo headInfo = headJsonObject.toJavaObject(HeadInfo.class);

        //返回主键ID
        Long id = crHeadInfService.saveHeadInfo(headInfo);
        commonParam.setHeadInfId(id);
    }

}
